module  climat_transient_GCM_mod

! Climate forcing in which we read a baseline climatology (e.g. preindustrial)
! on which we add transient anomalies
! Be careful the transient anomalies have to be computed from a climate model that have
! a constant topography since we account for the elevation feedback computed by GRISLI, 
! but not the one included in the climate anomalies forcing fields
  
  use module3d_phy,only:nx,ny,sealevel_2d,S,S0,Tmois,Tann,Tjuly,acc,num_forc,num_param,num_rep_42,ro,coefbmshelf,dt,time,dirforcage,dirnameinp
  use netcdf
  use io_netcdf_grisli

  implicit none
  
  real,dimension(:,:,:,:),allocatable   :: prec_anom  !< precip anomalies, indexes: nx,ny,mois,tsnap
  real,dimension(:,:,:,:),allocatable   :: tmoi_anom  !< Tann anomalies,   indexes: nx,ny,mois,tsnap
  real,dimension(:),allocatable       :: time_snap    !< snapshots dates   indexes: nb_snap

  real,dimension(nx,ny,12)    :: t2m0         !< reference period temperature
  real,dimension(nx,ny,12)    :: pr0          !< reference period temperature
  real,dimension(nx,ny,12) :: prmois ! precip sur topo Zs
  real,dimension(nx,ny) :: orog0     !< topo de la climato
  
  real :: coef_prec_unit     !< conversion factor for the precip
  real :: psolid=2.                              !< temp limit between liquid and solid precip
  real :: lapserate                              !< vertical temperature gradient
  real :: rappact                                !< Clausius-Clapeyron factor for precipitation change

  integer  :: massb_time        !< to choose if we actually apply the anomalies or not

  integer  :: nb_snap           !< number of snapshots
  
contains
  

subroutine input_clim  ! here we read the baseline climatology (e.g. preindustrial)

  implicit none
  
  real,dimension(:,:,:),allocatable   :: prec_anominit  !< precip anomalies, indexes: nx,ny,mois*tsnap
  real,dimension(:,:,:),allocatable   :: tmoi_anominit  !< Tann anomalies,   indexes: nx,ny,mois*tsnap
  real,dimension(:),allocatable       :: time_snapinit  !< snapshots dates   indexes: nb_snap*12
  
  ! for ncdf reading:
  real*8, dimension(:), pointer          :: tab1D             !< tableau 1d real pointer
  real*8, dimension(:,:), pointer        :: tab2D             !< tableau 2d real pointer
  real*8, dimension(:,:,:), pointer      :: tab3D             !< tableau 3d real pointer
  real*8, dimension(:,:,:,:), pointer    :: tab4D             !< tableau 4d real pointer
  
  real               :: time_depart_snaps  !> temps du debut premier snapshot
  real               :: ecart_snap  !> ecart entre 2 snapshots
  
  character(len=100) :: clim_ref_file, clim_anom_file
  logical :: flag_monthly

  integer :: err                ! recuperation d'erreur
  integer :: m,nbmois,k
  
  namelist/clim_transient_GCM/clim_ref_file,flag_monthly,coef_prec_unit,lapserate,rappact
  namelist/anom_transient_GCM/clim_anom_file,nb_snap,time_depart_snaps,ecart_snap,massb_time

  rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
  read(num_param,clim_transient_GCM)

  
  write(num_rep_42,'(A)')'!___________________________________________________________' 
  write(num_rep_42,'(A)') '&clim_transient_GCM                                ! nom du bloc '
  write(num_rep_42,*)
  write(num_rep_42,'(A,A,A)') 'clim_ref_file = "',trim(clim_ref_file),'"'
  write(num_rep_42,*) 'flag_monthly = ', flag_monthly
  write(num_rep_42,*) 'coef_prec_unit = ', coef_prec_unit
  write(num_rep_42,*) 'lapserate = ', lapserate  
  write(num_rep_42,*) 'rappact = ', rappact
  write(num_rep_42,*)'/'                            
  write(num_rep_42,*)

  if (flag_monthly) then

     nbmois=12
     
! lecture du climat de reference : climato mensuelle
     print*,'lecture du climat de reference: ',trim(DIRNAMEINP)//TRIM(clim_ref_file)
     call Read_Ncdf_var('precip',trim(DIRNAMEINP)//TRIM(clim_ref_file),tab3d)
     pr0(:,:,:)=tab3d(:,:,:)*coef_prec_unit
     call Read_Ncdf_var('t2m',trim(DIRNAMEINP)//TRIM(clim_ref_file),tab3d)
     t2m0(:,:,:)=tab3d(:,:,:)
     call Read_Ncdf_var('orog',trim(DIRNAMEINP)//TRIM(clim_ref_file),tab2d)
     orog0(:,:)=tab2d(:,:)
  
! put back climate fields on the topography of the model:
     do m=1,12
        Tmois(:,:,m)=t2m0(:,:,m)-lapserate*(orog0(:,:)-S0(:,:))
        prmois(:,:,m)=pr0(:,:,m)*exp(rappact*(Tmois(:,:,m)-t2m0(:,:,m)))
     enddo

! the reference temperature and precip are now corrected for the surface elevation differences     
     t2m0(:,:,:) = Tmois(:,:,:)
     pr0(:,:,:)  = prmois(:,:,:)
     
     Tann=sum(Tmois,dim=3)/12.
     Tjuly=Tmois(:,:,7)

! snow accumulation:
     acc(:,:)=sum(prmois,dim=3,mask=Tmois < psolid)
     acc(:,:)=acc(:,:)*1000./ro  ! conversion en glace

! Note: Tmois and Acc are the variables used by the ablation routine

  else !annual

     nbmois=1
     
     write(*,*) "climat_transient_GCM not coded with annual field yet!"
     STOP 
  end if

428 format(A)

  ! ______ Anomalies de SMB (simule asmb initMIP) 
  rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
  read(num_param,anom_transient_GCM)

  write(num_rep_42,428)'!____________________________________________________________________________' 
  write(num_rep_42,428)'!  module climat_transient_GCM_mod                                             '
  write(num_rep_42,anom_transient_GCM)
  write(num_rep_42,428)'! clim_anom_file = fichier anomalies T et P                                   '
  write(num_rep_42,428)'! nb_snap       = nombre de snapshots                                        '
  write(num_rep_42,428)'! time_depart_snaps = debut du forçage                                       '
  write(num_rep_42,428)'! ecart_snap = ecart entre les snapshots, en annees                          '
  write(num_rep_42,428)'! massb_time = 0:fixe, 1:anomalies                                           '
  write(num_rep_42,428)'!____________________________________________________________________________'
  
  clim_anom_file  = trim(dirnameinp)//trim(clim_anom_file)

  if (massb_time.eq.1) then
     !dynamic allocation of time_snapinit
     if (allocated(time_snapinit)) then 
        deallocate(time_snapinit,stat=err)
        if (err/=0) then
           print *,"Erreur à la desallocation de time_snapinit",err
           stop 
        end if
     end if
     allocate(time_snapinit(nbmois*nb_snap),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau time_snapinit",err
        print *,"nb_snap = ",nb_snap
        stop 
     end if
     !dynamic allocation of prec_anominit
     if (allocated(prec_anominit)) then 
        deallocate(prec_anominit,stat=err)
        if (err/=0) then
           print *,"Erreur à la desallocation de prec_anominit",err
           stop 
        end if
     end if
     allocate(prec_anominit(nx,ny,nbmois*nb_snap),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau prec_anominit",err
        print *,"nx,ny,nb_snap = ",nx,',',ny,',',nb_snap
        stop 
     end if
     !dynamic allocation of tmoi_anominit
     if (allocated(tmoi_anominit)) then 
        deallocate(tmoi_anominit,stat=err)
        if (err/=0) then
           print *,"Erreur à la desallocation de tmoi_anominit",err
           stop 
        end if
     end if
     allocate(tmoi_anominit(nx,ny,nbmois*nb_snap),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau tmoi_anom",err
        print *,"nx,ny,nb_snap = ",nx,',',ny,',',nb_snap
        stop 
     end if

! reading climatic anomalies
     call Read_Ncdf_var('prec_anomaly',clim_anom_file,tab3D)
     prec_anominit(:,:,:) = tab3D(:,:,:)
     call Read_Ncdf_var('tmois_anomaly',clim_anom_file,tab3D)
     tmoi_anominit(:,:,:) = tab3D(:,:,:)     
! reading date of the anomalies
     call Read_Ncdf_var('time',clim_anom_file,tab1D)
     time_snap(:) = tab1D(:)
     time_snap(:) = time_snap(:) + time_depart_snaps
     
! anomalies with the right dimensions (nx,ny,month,year):
     !dynamic allocation of time_snap
     if (allocated(time_snap)) then 
        deallocate(time_snap,stat=err)
        if (err/=0) then
           print *,"Erreur à la desallocation de time_snap",err
           stop 
        end if
     end if
     allocate(time_snap(nb_snap),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau time_snap ",err
        print *,"nb_snap = ",nb_snap
        stop 
     end if
     !dynamic allocation of prec_anom
     if (allocated(prec_anom)) then 
        deallocate(prec_anom,stat=err)
        if (err/=0) then
           print *,"Erreur à la desallocation de prec_anom",err
           stop 
        end if
     end if
     allocate(prec_anom(nx,ny,nbmois,nb_snap),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau prec_anom",err
        print *,"nx,ny,nb_snap = ",nx,',',ny,',',nb_snap
        stop 
     end if
     !dynamic allocation of tmoi_anom
     if (allocated(tmoi_anom)) then 
        deallocate(tmoi_anom,stat=err)
        if (err/=0) then
           print *,"Erreur à la desallocation de tmoi_anom",err
           stop 
        end if
     end if
     allocate(tmoi_anom(nx,ny,nbmois,nb_snap),stat=err)
     if (err/=0) then
        print *,"erreur a l'allocation du tableau tmoi_anom",err
        print *,"nx,ny,nb_snap = ",nx,',',ny,',',nb_snap
        stop 
     end if 
     if (flag_monthly) then
        do k=1,nb_snap
           do m=1,12
              prec_anom(:,:,m,k)=prec_anominit(:,:,m+(k-1)*12)
              tmoi_anom(:,:,m,k)=tmoi_anominit(:,:,m+(k-1)*12)
           enddo
           time_snap(k)= time_depart_snaps+(k-1)*ecart_snap
        enddo
        deallocate(prec_anominit)
        deallocate(tmoi_anominit)
     else
        write(*,*) "yearly forcage GCM not implemented yet"
        STOP
     endif
     
     
  endif

  !write(*,*) "input_clim ok, time in snapshots:", time_snap(:)
  
end subroutine input_clim
  
subroutine init_forclim

  implicit none
  
end subroutine init_forclim


subroutine forclim               !  au temps considere (time) 

  implicit none


  real,dimension(nx,ny,12) :: prec_anomtime, tmoi_anomtime
  real,dimension(nx,ny) :: Zs        !< surface topography above sea level
  real,dimension(nx,ny) :: deltaZs   !< topography diff with respect to reference topography

  double precision :: time_prec
  integer :: k,m
  
! get prec and tmois between two snapshots
! before -> first anom value
! after -> last value

  if(time.lt.time_snap(1)) then              ! time before forcing fields
     prec_anomtime(:,:,:) = prec_anom(:,:,:,1)
     tmoi_anomtime(:,:,:) = tmoi_anom(:,:,:,1)
     time_prec    = time
  else if (time.ge.time_snap(nb_snap)) then  ! time after forcing fields
     prec_anomtime(:,:,:) = prec_anom(:,:,:,nb_snap)
     tmoi_anomtime(:,:,:) = tmoi_anom(:,:,:,nb_snap)
     if (abs(time-time_prec-1.).lt.dt) then   
        time_prec = time_prec + 1
     endif
  else                                       ! general case
     do k = 1 , nb_snap-1
        if((time.ge.time_snap(k)).and.(time.lt.time_snap(k+1))) then ! between k and k+1 

           prec_anomtime(:,:,:) = prec_anom(:,:,:,k) + (prec_anom(:,:,:,k+1)-prec_anom(:,:,:,k)) *   &
                (time-time_snap(k))/(time_snap(k+1)-time_snap(k))
           tmoi_anomtime(:,:,:) = tmoi_anom(:,:,:,k) + (tmoi_anom(:,:,:,k+1)-tmoi_anom(:,:,:,k)) *   &
                (time-time_snap(k))/(time_snap(k+1)-time_snap(k))
           if ((abs(time-time_snap(k)).le.dt).and.(abs(time-time_prec-1.).lt.dt)) then    
              time_prec = time_snap(k)     ! time_prec est le temps du precedent
           endif
           exit
        endif
     end do
  endif


  
! put back climate fields on the topography of the model:
  Zs(:,:)=max(sealevel_2d(:,:),S(:,:))
  deltaZs(:,:)= -lapserate*(Zs(:,:)-S0(:,:)) ! temperature difference due to surface elevation with respect to reference period
  do m=1,12
     Tmois(:,:,m)=t2m0(:,:,m)+tmoi_anomtime(:,:,m)+deltaZs(:,:)
     prmois(:,:,m)=pr0(:,:,m)*prec_anomtime(:,:,m)*exp(rappact*(deltaZs(:,:))) ! /!\ we take only deltaZs because the anom of precip computed
                                                          !     from the GCM already accounts for temperature change
  enddo

  Tann=sum(Tmois,dim=3)/12.
  Tjuly=Tmois(:,:,7)

! snow accumulation:
  acc(:,:)=sum(prmois,dim=3,mask=Tmois < psolid)
  acc(:,:)=acc(:,:)*1000./ro  ! conversion en glace


  
end subroutine forclim

end module climat_transient_GCM_mod
